'use strict'
const yeoman = require('yeoman-generator')
const chalk = require('chalk')

function hubotStartSay () {
  return '                     _____________________________  ' + '\n' +
          '                    /                             \\ ' + '\n' +
          ' ' + chalk.cyan('  //\\') + '              |      Extracting input for    |' + '\n' +
          ' ' + chalk.cyan(' ////\\  ') + '  ' + chalk.yellow('_____') + '    |   script generation process  |' + '\n' +
          ' ' + chalk.cyan('//////\\  ') + chalk.yellow('/') + chalk.cyan('_____') + chalk.yellow('\\') + '   \\                             / ' + '\n' +
          ' ' + chalk.cyan('=======') + chalk.yellow(' |') + chalk.cyan('[^_/\\_]') + chalk.yellow('|') + '   /----------------------------  ' + '\n' +
          '  ' + chalk.yellow('|   | _|___') + '@@' + chalk.yellow('__|__') + '                                ' + '\n' +
          '  ' + chalk.yellow('+===+/  ///     ') + chalk.cyan('\\_\\') + '                               ' + '\n' +
          '   ' + chalk.cyan('| |_') + chalk.yellow('\\ /// HUBOT/') + chalk.cyan('\\\\') + '                             ' + '\n' +
          '   ' + chalk.cyan('|___/') + chalk.yellow('\\//      /') + chalk.cyan('  \\\\') + '                            ' + '\n' +
          '         ' + chalk.yellow('\\      /   +---+') + '                            ' + '\n' +
          '          ' + chalk.yellow('\\____/    |   |') + '                            ' + '\n' +
          '           ' + chalk.cyan('| //|') + '    ' + chalk.yellow('+===+') + '                            ' + '\n' +
          '            ' + chalk.cyan('\\//') + '      |xx|                            ' +
          '\n'
}

function hubotEndSay () {
  return '                     _____________________________  ' + '\n' +
          ' _____              /                             \\ ' + '\n' +
          ' \\    \\             |   Script generation process   |' + '\n' +
          ' |    |    ' + chalk.yellow('_____') + '    |          complete...         |' + '\n' +
          ' |__' + chalk.cyan('\\\\') + '|   ' + chalk.yellow('/') + chalk.cyan('_____') + chalk.yellow('\\') + '   \\     Good luck with that.    / ' + '\n' +
          '   ' + chalk.cyan('|//') + chalk.yellow('+  |') + chalk.cyan('[^_/\\_]') + chalk.yellow('|') + '   /----------------------------  ' + '\n' +
          '  ' + chalk.yellow('|   | _|___') + '@@' + chalk.yellow('__|__') + '                                ' + '\n' +
          '  ' + chalk.yellow('+===+/  ///     ') + chalk.cyan('\\_\\') + '                               ' + '\n' +
          '   ' + chalk.cyan('| |_') + chalk.yellow('\\ /// HUBOT/') + chalk.cyan('\\\\') + '                             ' + '\n' +
          '   ' + chalk.cyan('|___/') + chalk.yellow('\\//      /') + chalk.cyan('  \\\\') + '                            ' + '\n' +
          '         ' + chalk.yellow('\\      /   +---+') + '                            ' + '\n' +
          '          ' + chalk.yellow('\\____/    |   |') + '                            ' + '\n' +
          '           ' + chalk.cyan('| //|') + '    ' + chalk.yellow('+===+') + '                            ' + '\n' +
          '            ' + chalk.cyan('\\//') + '      |xx|                            ' +
          '\n'
}

function extractScriptName (_, appname) {
  const slugged = _.slugify(appname)
  const match = slugged.match(/^hubot-(.+)/)
  if (match && match.length === 2) {
    return match[1].toLowerCase()
  }
  return slugged
}

const HubotScriptGenerator = yeoman.generators.Base.extend({
  initializing: function () {
    this.pkg = require('../../package.json')
  },

  prompting: {
    askFor: function () {
      const done = this.async()
      const userName = this.user.git.name()
      const userEmail = this.user.git.email()

      const prompts = [{
        name: 'scriptOwner',
        message: 'Owner',
        default: userName + ' <' + userEmail + '>'
      }]

      this.log(hubotStartSay())

      this.prompt(prompts, function (props) {
        this.scriptOwner = props.scriptOwner

        done()
      }.bind(this))
    },

    askForScriptNameAndDescription: function () {
      const done = this.async()
      const scriptName = extractScriptName(this._, this.appname)

      const prompts = [{
        name: 'scriptName',
        message: 'Script name',
        default: scriptName
      },
      {
        name: 'scriptDescription',
        message: 'Description',
        default: 'A hubot script that does the things'
      },
      {
        name: 'scriptKeywords',
        message: 'Keywords',
        default: 'hubot, hubot-scripts'
      }]

      this.prompt(prompts, function (props) {
        this.scriptName = props.scriptName
        this.scriptDescription = props.scriptDescription
        this.scriptKeywords = JSON.stringify(props.scriptKeywords.split(/\s*,\s*/))
        this.appname = 'hubot-' + this.scriptName

        done()
      }.bind(this))
    }
  },

  writing: {
    app: function () {
      this.mkdir('script')
      this.copy('script/bootstrap', 'script/bootstrap')
      this.copy('script/test', 'script/test')

      this.mkdir('src')
      this.template('src/template.js', 'src/' + this.scriptName + '.js')

      this.mkdir('test')
      this.template('test/template-test.js', 'test/' + this.scriptName + '-test.js')

      this.copy('gitignore', '.gitignore')
      this.copy('.travis.yml', '.travis.yml')
      this.copy('index.js', 'index.js')
      this.template('_package.json', 'package.json')
      this.copy('README.md', 'README.md')
    }
  },

  end: function () {
    this.npmInstall()

    this.log(hubotEndSay())
  }
})

module.exports = HubotScriptGenerator
